--
-- $Id: Utils.hs,v 1.6 2005/12/03 19:17:24 aamine Exp $
--
-- Copyright (c) 2003-2005 Minero Aoki
--
-- This program is free software.
-- You can distribute/modify this program under the terms of
-- the GNU LGPL, Lesser General Public License version 2.1.
-- For details of the GNU LPGL, see the file "COPYING".
--

module Utils
    (dirEntries, makePath, myGetEnv, errorExit, packList)
where

import PathUtils

import System
import System.IO
import System.IO.Error
import System.Directory
import Control.Monad (when)

dirEntries :: FilePath -> IO [String]
dirEntries path = getDirectoryContents path >>= (return . filter notDotFile)
  where
    notDotFile ('.':_)  = False
    notDotFile _        = True

makePath :: FilePath -> IO ()
makePath = mkdirs . ancestors
  where
    mkdirs :: [String] -> IO ()
    mkdirs [] = return ()
    mkdirs (x:xs) =
        catch (mkdir_f x)
              (\ex -> if isDoesNotExistError ex then
                          mkdirs xs >>
                          mkdir_f x
                      else
                          ioError ex)

    mkdir_f path =
        catch (createDirectory path)
              (\ex -> if isAlreadyExistsError ex then
                          return ()
                      else
                          ioError ex)

myGetEnv :: String -> IO String
myGetEnv name =
    catch (getEnv name)
          (\ex -> errorExit $ "environment variable not set: " ++ name)

errorExit :: String -> IO a
errorExit msg = do hPutStrLn stderr msg
                   exitWith (ExitFailure 1)

packList :: Int -> [String] -> String
packList max = unlines . map (unwords) . makeMatrix (max+1)
  where
    makeMatrix :: Int -> [String] -> [[String]]
    makeMatrix max []          = []
    makeMatrix max list@(x:xs) = let (y,ys) = takeList max list
                                 in case y of
                                      [] -> [x] : makeMatrix max xs
                                      _  -> y   : makeMatrix max ys

    takeList :: Int -> [String] -> ([String], [String])
    takeList max xs = let (a,b) = break ((<) max . fst) $ zip (lengthSum xs) xs
                      in (map (snd) a, map (snd) b)

    lengthSum :: [String] -> [Int]
    lengthSum = scanl1 (+) . map (\s -> length s + 1)
