--
-- $Id: href.hs,v 1.23 2005/12/03 21:16:46 aamine Exp $
--
-- Copyright (c) 2003-2005 Minero Aoki
--
-- This program is free software.
-- You can distribute/modify this program under the terms of
-- the GNU LGPL, Lesser General Public License version 2.1.
-- For details of the GNU LGPL, see the file "COPYING".
--

module Main (main) where

import HrefDatabase
import Utils

import System.IO
import System.Environment
import System.Exit
import Data.List
import Data.Char (isUpper, toLower)
import Data.Maybe

main = do prefix <- myGetEnv hrefDataDirEnv
          args <- getArgs
          case args of
            []         -> putPackedList =<< readModules prefix
            [name]     -> if seemsModuleName name then
                              lookupModule prefix name
                          else
                              lookupFunction prefix "" name
            [mod,func] -> lookupFunction prefix mod func
            _          -> do hPutStrLn stderr "too many arguments"
                             exitWith $ ExitFailure 1

seemsModuleName "" = False
seemsModuleName name = (isUpper $ head name) || (elem '.' name)

lookupModule prefix pattern =
    do names <- readModules prefix
       case match pattern (sort names) of
         []    -> notFound "module" pattern
         [mod] -> do putStrLn $ "---- module " ++ mod
                     putPackedList =<< readModuleFunctions prefix mod
         mods  -> putPackedList mods
  where
    match pattern = filter (ncIsPrefixOf pattern)

lookupFunction prefix mod func =
    do specs <- readSpecs prefix
       case best $ applyFilters (makeFilters mod func) $ specs of
         Just [f] -> describe f
         Just fs  -> putList $ map (show) fs
         Nothing  -> notFound "function" (mod ++ " " ++ func)
  where
    describe spec = do putStrLn (show spec)
                       putStr =<< readDocument prefix spec

best sets = (find (just1) sets) `onFail` (shortest sets)
  where
    onFail a b = if isJust a then a else b

    just1 [x] = True
    just1 _   = False

    shortest = listToMaybe . filter (not.null) . sortBy (shorterFirst)

    shorterFirst a b = length b `compare` length a

applyFilters fs specs = map ($ specs) fs

makeFilters mod func = map (\m -> filter (specMatchBy mod func m)) methods
  where
    methods = [(ncIsPrefixOf), (isPrefixOf), (ncEqual), (==)]

ncEqual a b = downcase a == downcase b

ncIsPrefixOf a b = downcase a `isPrefixOf` downcase b

downcase = map (toLower)

putPackedList items =
    do ok <- hIsTerminalDevice stdout
       ncols <- getTermColumns
       putStr $ if ok
                then packList ncols $ sort items
                else unlines $ sort items

putList = putStr . unlines . sort

defaultColumns = 70

getTermColumns :: IO Int
getTermColumns = catch (return . read =<< getEnv "COLUMNS")
                       (\ex -> return defaultColumns)

notFound kind name = errorExit $ "no such " ++ kind ++ ": " ++ name
